/**
 * 月日を数値形式に変換する関数
 * @param month - 月 (1-12)
 * @param day - 日 (1-31)
 * @returns 月日を組み合わせた数値 (例: 1月1日 → 101, 12月1日 → 1201)
 */
export function dateToNumber(month: number, day: number): number {
    // 入力値の検証
    if (month < 1 || month > 12) {
        throw new Error('月は1から12の間で指定してください');
    }
    if (day < 1 || day > 31) {
        throw new Error('日は1から31の間で指定してください');
    }

    // 月が1桁の場合は1桁のまま、2桁の場合は2桁で表示
    const monthStr = month.toString();
    // 日は必ず2桁で表示（1桁の場合は0埋め）
    const dayStr = day.toString().padStart(2, '0');

    return parseInt(monthStr + dayStr);
}

// 使用例
/*
console.log(dateToNumber(1, 1));   // 101
console.log(dateToNumber(3, 15));  // 315
console.log(dateToNumber(12, 1));  // 1201
*/ 