from typing import TypeVar, List

T = TypeVar('T')  # 任意の型を表すための型変数

def flatten_list(nested_list: List[List[T]]) -> List[T]:
    """
    同種の（homogeneous）ネストされたリストをフラットにする関数

    Args:
        nested_list (List[List[T]]): ネストされたリスト

    Returns:
        List[T]: フラット化されたリスト

    Examples:
        >>> flatten_list([[1, 2], [3, 4]])
        [1, 2, 3, 4]
        >>> flatten_list([['a', 'b'], ['c']])
        ['a', 'b', 'c']
    """
    return [item for sublist in nested_list for item in sublist]


# 使用例
if __name__ == '__main__':
    # 数値のリスト
    nested_numbers = [[1, 2, 3], [4, 5], [6]]
    flat_numbers = flatten_list(nested_numbers)
    print("数値リストの例:")
    print(f"元のリスト: {nested_numbers}")
    print(f"フラット化: {flat_numbers}")  # [1, 2, 3, 4, 5, 6]

    # 文字列のリスト
    nested_strings = [['hello', 'world'], ['python'], ['coding']]
    flat_strings = flatten_list(nested_strings)
    print("\n文字列リストの例:")
    print(f"元のリスト: {nested_strings}")
    print(f"フラット化: {flat_strings}")  # ['hello', 'world', 'python', 'coding'] 